function love.load(arg)
  if arg[#arg] == "-debug" then require("mobdebug").start() end  
  
  love.window.setMode(48, 48)
  
  p = {}
  p.image = love.graphics.newImage("player.png")
  
  -- Coords for primary image
  p.x = love.graphics.getWidth()  / 2
  p.y = love.graphics.getHeight() / 2
  
  p.w = p.image:getWidth()
  p.h = p.image:getHeight()
  p.ox = p.w / 2
  p.oy = p.h / 2
  p.a = 0           -- angle
  
  -- Coords for secondary images used for border crossing or wrapping
  -- This table is empty when no part of the player image is wrapped
  -- around any window border.
  p.wrap_coords = {}
end

function isVisible(x, y, ox, oy)
  local wh = love.graphics.getHeight()
  local ww = love.graphics.getWidth()
  
  return  x + ox >= 0 or
          x - ox < ww or
          y + oy >= 0 or
          y - oy < wh
end


function love.update(dt)
  local wh = love.graphics.getHeight()
  local ww = love.graphics.getWidth()
  local wrap_coord_offsets = 
  {
    {x = 0,   y = -wh}, -- top
    {x = ww,  y = -wh}, -- top right
    {x = ww,  y = 0},   -- right
    {x = ww,  y = wh},  -- bottom right
    {x = 0,   y = wh},  -- bottom
    {x = -ww, y = wh},  -- bottom left
    {x = -ww, y = 0},   -- left
    {x = -ww, y = -wh}  -- top left
  }
  
  -- Fix up the angle to be [0..2pi]
  if p.a < 0 then
    p.a = 2 * math.pi + p.a
  elseif p.a > 2 * math.pi then
    p.a = p.a - 2 * math.pi
  end
  
  -- Clear wrapped image coordinates. They are about to be recalculated.
  for i, wco in ipairs(p.wrap_coords) do
    p.wrap_coords[i] = nil
  end
  
  -- Calculate coordinates of all 8 images used for wrapping around borders
  -- Only add to p.wrap_coords if the image is visible
  for i, wco in ipairs(wrap_coord_offsets) do
    local x = p.x + wco.x
    local y = p.y + wco.y
    if isVisible(x, y, p.ox, p.oy) then
      -- Whichever one of these has its coords inside the window becomes the
      -- primary image. Don't add it to the secondary wrap images
      if x >= 0 and x < ww and y >= 0 and y < wh then
        p.x = x
        p.y = y
      else
        table.insert(p.wrap_coords, {x = x, y = y})
      end
    end
  end
end

function love.draw()
  -- Rotate player image around its center.
  -- This moves the origin to the center of the image
  love.graphics.draw(p.image, p.x, p.y, p.a, 1, 1, p.ox, p.oy)
  
  -- Draw any boundary wrapping images
  for i, wc in ipairs(p.wrap_coords) do
    love.graphics.draw(p.image, wc.x, wc.y, p.a, 1, 1, p.ox, p.oy)
  end
  
  love.graphics.print(p.x .. "," .. p.y)
end

function love.keypressed(key)
  if key == "w" then
    p.y = p.y - 1
  elseif key == "s" then
    p.y = p.y + 1
  elseif key == "a" then
    p.x = p.x - 1
  elseif key == "d" then
    p.x = p.x + 1
  elseif key == "q" then
    p.a = p.a - math.pi / 32
  elseif key == "e" then
    p.a = p.a + math.pi / 32
  end
end
